#!/usr/bin/bash

# 列出已加载的模块
loaded_modules=$(lsmod | awk '{print $1}' | tail -n +2)

has_third_party_modules=false

# 检查每个模块的来源
for module in $loaded_modules; do
    modinfo_output=$(modinfo $module)
    filename=$(echo "$modinfo_output" | grep "^filename:" | awk '{print $2}')
    intree=$(echo "$modinfo_output" | grep "^intree:" | awk '{print $2}')

    if [ "$intree" != "Y" ]; then
        package=""
        if command -v dpkg &> /dev/null; then
            package=$(dpkg -S "$filename" 2>/dev/null)
        elif command -v rpm &> /dev/null; then
            package=$(rpm -qf "$filename" 2>/dev/null)
        fi

        if [ -z "$package" ]; then
            echo "Module: $module"
            echo "Filename: $filename"
            echo "Source: Possibly user-installed"
            echo "-----------------------------"
            has_third_party_modules=true
        fi
    fi
done

if [ "$has_third_party_modules" = true ]; then
    #echo "Warning: Third-party modules detected. Please check the source of these modules."
    exit 1
else
    #echo "No third-party modules detected."
    exit 0
fi
